package dk.nsp.idsas.integrations.sores;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.StringEntity;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.time.Duration;

public class SoresClient  {
    public static final String ERROR_MSG = "Fejl i SORES-integrationen";
    public static final String ERROR_MSG_BAD_REQUEST = "SORES svarede med status 400";
    public static final String ERROR_MSG_INTERNAL_SERVER_ERROR = "SORES svarede med status 500";

    private static final String CONTENT_TYPE = "application/json";

    private final ObjectMapper objectMapper = new ObjectMapper();
    private final String endpoint;
    private final HttpClient httpClient;
    private final ServiceErrorCollector serviceErrorCollector;

    public SoresClient(HttpClient httpClient, String endpoint, ServiceErrorCollector serviceErrorCollector) {
        this.endpoint = endpoint;
        this.httpClient = httpClient;
        this.serviceErrorCollector = serviceErrorCollector;
    }

    public SoresClient(HttpClient httpClient, String endpoint, Duration errorCountDuration) {
        this(httpClient, endpoint, new ServiceErrorCollector(errorCountDuration));
    }

    public int getErrorCount() {
        return serviceErrorCollector.getErrorCount();
    }

    public boolean checkSorExists(String sorCode) throws SoresClientException {
        try {
            HttpUriRequest request = buildSorExistsRequest(sorCode);
            boolean result = invoke(sorCode, request);
            serviceErrorCollector.registerServiceSuccess();
            return result;
        } catch (JsonProcessingException | UnsupportedOperationException | UnsupportedEncodingException e) {
            serviceErrorCollector.registerServiceError();
            throw new SoresClientException(ERROR_MSG, e);
        }
    }

    private HttpUriRequest buildSorExistsRequest(String sorCode) throws JsonProcessingException, UnsupportedEncodingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String jsonString = objectMapper.writeValueAsString(new Object() {
            public final String op = "GetSorEntity";
            public final String[] sorIds = { sorCode };
        });
        final StringEntity entity = new StringEntity(jsonString);

        final HttpPost httpPost = new HttpPost(endpoint);
        httpPost.setEntity(entity);
        httpPost.setHeader("Content-type", CONTENT_TYPE);
        return httpPost;
    }

    private boolean invoke(String sorCode, HttpUriRequest request) throws SoresClientException {
        try {
            HttpResponse response = httpClient.execute(request);
            return handleResponse(sorCode, response);
        } catch (IOException e) {
            serviceErrorCollector.registerServiceError();
            throw new SoresClientException(ERROR_MSG, e);
        }
    }

    private boolean handleResponse(String sorCode, HttpResponse response) throws SoresClientException, IOException {
        if (response.getStatusLine().getStatusCode() == HttpStatus.SC_INTERNAL_SERVER_ERROR) {
            serviceErrorCollector.registerServiceError();
            throw new SoresClientException(ERROR_MSG_INTERNAL_SERVER_ERROR);
        }

        if (response.getStatusLine().getStatusCode() == HttpStatus.SC_BAD_REQUEST) {
            serviceErrorCollector.registerServiceError();
            throw new SoresClientException(ERROR_MSG_BAD_REQUEST);
        }

        JsonNode jsonNode = objectMapper.readTree(response.getEntity().getContent());
        jsonNode = jsonNode.get(sorCode);
        return !jsonNode.isEmpty();
    }
}
