package dk.nsp.idsas.integrations.sores;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpUriRequest;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.io.IOException;

import static dk.nsp.idsas.integrations.sores.TestUtil.createHttpResponse;
import static org.junit.Assert.*;

public class SoresClientTest {
    private HttpClient httpClient;
    private final String url = "test-url";

    private ServiceErrorCollector serviceErrorCollector;
    private SoresClient soresClient;

    @Before
    public void setUp() {
        httpClient = Mockito.mock(HttpClient.class);
        serviceErrorCollector = Mockito.mock(ServiceErrorCollector.class);
        soresClient = new SoresClient(httpClient, url, serviceErrorCollector);
    }

    private String generateValidJSONResponse(String sorCode) {
        return String.format("{\n" +
                "  \"%s\": {\n" +
                "    \"sorId\": %s,\n" +
                "    \"from\": \"2022-03-19\",\n" +
                "    \"name\": \"ADA Psykiatrisk Udredning - AUH\",\n" +
                "    \"healthInstitutionSorId\": 381101000016008,\n" +
                "    \"shakId\": \"660051Y\",\n" +
                "    \"parentSorId\": 694881000016003\n" +
                "  }\n" +
                "}", sorCode, sorCode);
    }

    private String generateEmptyJSONResponse(String sorCode) {
        return String.format("{\n" +
                "    \"%s\": {}\n" +
                "}", sorCode);
    }

    @Test
    public void sores_WhenServiceReturnsValidResponse_ReturnsCvrAndRegistersServiceSuccess() throws IOException, SoresClientException {
        // Arrange
        String sorCode = "1234";
        HttpResponse httpResponse = createHttpResponse(200, "application/json", generateValidJSONResponse(sorCode));
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        boolean returnCvr = soresClient.checkSorExists(sorCode);

        // Assert
        assertTrue(returnCvr);
        String expectedPath = url;
        Mockito.verify(httpClient).execute(
                Mockito.argThat(request -> expectedPath.equals(request.getURI().getPath())));
        Mockito.verify(serviceErrorCollector).registerServiceSuccess();
    }

    @Test
    public void sores_WhenServiceReturnsEmptyResponse_ReturnsFalseAndRegisterServiceSuccess()  throws IOException, SoresClientException {
        // Arrange
        String sorCode = "1234";
        HttpResponse httpResponse = createHttpResponse(200, "application/json", generateEmptyJSONResponse(sorCode));
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        boolean returnCvr = soresClient.checkSorExists(sorCode);

        // Assert
        assertFalse(returnCvr);
        String expectedPath = url;
        Mockito.verify(httpClient).execute(
                Mockito.argThat(request -> expectedPath.equals(request.getURI().getPath())));
        Mockito.verify(serviceErrorCollector).registerServiceSuccess();
    }

    @Test
    public void sores_WhenClientThrowsIOException_ThrowsSoresClientExceptionAndRegisterError() throws IOException {
        // Arrange
        Mockito.doThrow(IOException.class).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.checkSorExists("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG));
    }

    @Test
    public void sores_WhenServiceReturnsResponse200WithInvalidJson_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        // Arrange
        HttpResponse httpResponse = createHttpResponse(200, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.checkSorExists("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG));
    }

    @Test
    public void sores_WhenServiceReturnsResponse404WithInvalidJson_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        // Arrange
        HttpResponse httpResponse = createHttpResponse(404, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.checkSorExists("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG));
    }

    @Test
    public void sores_WhenServiceReturnsResponse400_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        // Arrange
        HttpResponse httpResponse = createHttpResponse(400, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.checkSorExists("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG_BAD_REQUEST));
    }

    @Test
    public void sores_WhenServiceReturnsResponse500_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        // Arrange
        HttpResponse httpResponse = createHttpResponse(500, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.checkSorExists("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG_INTERNAL_SERVER_ERROR));
    }
}