package dk.nsp.idsas.ws._20240124;

import com.sun.xml.ws.developer.SchemaValidation;
import dk.nsp.idsas.dto.OrganizationIdDto;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.security.IdsasSecurityException;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.service.IdsasServiceException;
import dk.nsp.idsas.service.OrgBlurringService;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.FaultCode;
import dk.nsp.idsas.util.IdsasException;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2024._01._24.*;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;
import java.time.ZonedDateTime;
import java.util.List;
import java.util.stream.Collectors;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name="IdsasRegistration20240124", urlPatterns={"/20240124/registration"})
@WebService(serviceName = "IdsasRegistration20240124", wsdlLocation = "wsdl/20240124/idsas-registration.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2024/01/24/",
        portName = "IdsasRegistrationPort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation
public class IdsasRegistrationWS extends IdsasAbstractWS implements IdsasRegistrationPortType {
    private static final Logger logger = Logger.getLogger(IdsasRegistrationWS.class);

    @Autowired
    private IdsasActorBuilder idsasActorBuilder;

    @Autowired
    private BlurringService blurringService;

    @Autowired
    private OrgBlurringService orgBlurringService;

    @Override
    public String createBlurring(CreateBlurringRequest createBlurringRequest) {
        logger.trace("Enter create blurring");

        try {
            IdsasActor actor = idsasActorBuilder.buildActor();
            String organizationId = actor.getOrganizationId();
            String organizationIdType = actor.getOrganizationIdType();
            String patientIdType = createBlurringRequest.getPatientId().getClassification().value();
            String patientId = createBlurringRequest.getPatientId().getId();
            ZonedDateTime endDateTime = createBlurringRequest.getEndDateTime().toGregorianCalendar().toZonedDateTime();

            blurringService.createBlurring(organizationId, organizationIdType, patientId, patientIdType, endDateTime);
        } catch (IdsasException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        } catch (Exception e) {
            logger.error("Failed to create blurring", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error",
                    FaultCode.CREATE_BLURRING_ERROR.getValue());
        }

        return "OK";
    }

    @Override
    public String createOrgBlurring(CreateOrgBlurringRequest createOrgBlurringRequest) {
        IdsasActor actor = getOrgBlurringActor();

        try {
            orgBlurringService.createOrgBlurring(actor.getOrganizationId(),
                    createOrgBlurringRequest.getOrg().getId(), createOrgBlurringRequest.getOrg().getClassification().value());
        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }

        return "OK";
    }

    @Override
    public ListOrgBlurringsForCVRResponse listOrgBlurringsForCVR(ListOrgBlurringsForCVRRequest emptyInput) {
        IdsasActor actor = getOrgBlurringActor();

        try {
            List<OrganizationIdDto> orgs = orgBlurringService.listOrgBlurringsForCVR(actor.getOrganizationId());
            List<BlurredOrgIdType> orgIdTypes = orgs.stream()
                    .map(o -> {
                        BlurredOrgIdType orgIdType = new BlurredOrgIdType();
                        BlurredOrgIdClassificationType classification = BlurredOrgIdClassificationType.fromValue(o.idType.name());
                        orgIdType.setId(o.id);
                        orgIdType.setClassification(classification);
                        return orgIdType;
                    })
                    .collect(Collectors.toList());

            ListOrgBlurringsForCVRResponse response = new ListOrgBlurringsForCVRResponse();
            response.getOrg().addAll(orgIdTypes);
            return response;

        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }
    }

    @Override
    public ListAllActiveOrgBlurringsResponse listAllActiveOrgBlurrings(ListAllActiveOrgBlurringsRequest emptyInput) {
        try {
            List<OrganizationIdDto> orgs = orgBlurringService.listAllActiveOrgBlurrings();
            List<BlurredOrgIdType> orgIdTypes = orgs.stream()
                    .map(o -> {
                        BlurredOrgIdType orgIdType = new BlurredOrgIdType();
                        BlurredOrgIdClassificationType classification = BlurredOrgIdClassificationType.fromValue(o.idType.name());
                        orgIdType.setId(o.id);
                        orgIdType.setClassification(classification);
                        return orgIdType;
                    })
                    .collect(Collectors.toList());

            ListAllActiveOrgBlurringsResponse response = new ListAllActiveOrgBlurringsResponse();
            response.getOrg().addAll(orgIdTypes);
            return response;

        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }
    }

    @Override
    public String removeOrgBlurring(RemoveOrgBlurringRequest removeOrgBlurringRequest) {
        IdsasActor actor = getOrgBlurringActor();

        try {
            orgBlurringService.removeOrgBlurring(actor.getOrganizationId(), removeOrgBlurringRequest.getId(), removeOrgBlurringRequest.getClassification().value());
        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }

        return "OK";
    }

    private IdsasActor getOrgBlurringActor() {
        IdsasActor actor;
        try {
            actor = idsasActorBuilder.buildActor();
            if (!actor.getActorType().equals(IdsasActor.IdsasActorType.SYSTEM)) {
                throw new IdsasSecurityException(ErrorMessages.userTypeNotSupported(
                        actor.getActorType().getDisplayableText()), FaultCode.CREATE_ORG_BLURRING_ERROR, true);
            }
            if (!actor.getOrganizationIdType().equalsIgnoreCase("CVR")) {
                throw new IdsasSecurityException(ErrorMessages.organizationIdTypeNotSupported(
                        actor.getOrganizationIdType()), FaultCode.CREATE_ORG_BLURRING_ERROR, true);
            }
        } catch (IdsasException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }
        return actor;
    }
}
