package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClient;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClientException;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.HashingUtil;
import dk.nsp.idsas.util.WhitelistType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.PatientIdClassificationType;
import org.apache.log4j.Logger;
import dk.nsp.idsas.util.FaultCode;
import org.springframework.dao.DataAccessException;

import java.time.ZonedDateTime;

public class BlurringService {
    private static final Logger logger = Logger.getLogger(BlurringService.class);

    private final WhitelistDao whitelistDao;
    private final BlurringDao blurringDao;
    private final AuditLogger auditLogger;
    private final PersonInformationClient personInformationClient;
    private final String patientIdSaltPrefix;
    private final String patientIdSaltPostfix;
    private final int expiryMaxDays;

    public BlurringService(WhitelistDao whitelistDao, BlurringDao blurringDao, String patientIdSaltPrefix, String patientIdSaltPostfix,
                           AuditLogger auditLogger, PersonInformationClient personInformationClient,
                           int expiryMaxDays) {
        this.whitelistDao = whitelistDao;
        this.blurringDao = blurringDao;
        this.auditLogger = auditLogger;
        this.personInformationClient = personInformationClient;
        this.patientIdSaltPrefix = patientIdSaltPrefix;
        this.patientIdSaltPostfix = patientIdSaltPostfix;
        this.expiryMaxDays = expiryMaxDays;
    }

    public void createBlurring(String organizationId, String organizationIdType, String patientId, String patientIdType, ZonedDateTime endDateTime) throws IdsasServiceException {

        // Tjek at organizationId er whitelisted til at kalde createBlurring
        boolean isWhiteListed = whitelistDao.isWhitelisted(organizationId, WhitelistType.BLURRING);
        if (!isWhiteListed) {
            throw new IdsasServiceException(ErrorMessages.cvrWhitelistError(organizationId, WhitelistType.BLURRING), FaultCode.CREATE_BLURRING_ERROR, true);
        }

        // Tjek at patientId er gyldig (via "personinformation")
        if (patientIdType.equals(PatientIdClassificationType.CPR.value())) {
            boolean patientIdIsValid = false;
            try {
                patientIdIsValid = personInformationClient.personExists(patientId);
                if (!patientIdIsValid) {
                    throw new IdsasServiceException(ErrorMessages.cprIsNotValidError(patientId), FaultCode.CREATE_BLURRING_ERROR, true);
                }
            } catch (PersonInformationClientException e) {
                throw new IdsasServiceException(e.getMessage(), FaultCode.CREATE_BLURRING_ERROR, false, e);
            }
        }

        // Tjek at sløringen ikke ligger mere end 'expiryMaxDays' fremme i tiden
        if (endDateTime.isAfter(ZonedDateTime.now().plusDays(expiryMaxDays))) {
            throw new IdsasServiceException(ErrorMessages.maxExpiryDateExceeded(expiryMaxDays), FaultCode.CREATE_BLURRING_ERROR, true);
        }

        // PatientId hashes!
        String hashedPatientId = HashingUtil.createPatientIdHash(patientId, patientIdSaltPrefix, patientIdSaltPostfix);

        // Entity oprettes og gemmes
        BlurringEntity entity = new BlurringEntity(
                hashedPatientId,
                patientIdType,
                organizationId,
                organizationIdType,
                endDateTime.toInstant());

        try {
            blurringDao.insert(entity);
        } catch (DataAccessException e) {
            throw new IdsasServiceException(ErrorMessages.daoAccessError(), FaultCode.CREATE_BLURRING_ERROR, false, e);
        }

        // Audit logning
        auditLogger.logCreateBlurring(patientId, patientIdType, hashedPatientId, organizationId, organizationIdType, entity.getExpiryDate());
    }

}
