package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.OrgBlurringDao;
import dk.nsp.idsas.dao.entity.OrgBlurringEntity;
import dk.nsp.idsas.dto.OrganizationIdDto;
import dk.nsp.idsas.integrations.sores.SoresClient;
import dk.nsp.idsas.integrations.sores.SoresClientException;
import dk.nsp.idsas.util.ErrorMessages;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.dao.DataAccessException;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class OrgBlurringServiceTest {
    private SoresClient soresClient;
    private OrgBlurringDao orgBlurringDao;
    private OrgBlurringService orgBlurringService;

    @Before
    public void setUp() {
        soresClient = Mockito.mock(SoresClient.class);
        orgBlurringDao = Mockito.mock(OrgBlurringDao.class);
        orgBlurringService = new OrgBlurringService(orgBlurringDao, soresClient);
    }

    @Test
    public void createOrgBlurring_sorExistsRegistrationDoesNot_noErrors() throws IdsasServiceException, SoresClientException {
        // Act
        when(soresClient.checkSorExists("12345678")).thenReturn(true);
        when(orgBlurringDao.orgBlurringExists("123", "SOR")).thenReturn(false);
        String sor = "12345678";
        String cvr = "123";

        // Act
        orgBlurringService.createOrgBlurring(cvr, sor, "SOR");

        // Assert
        verify(soresClient).checkSorExists(sor);
        verify(orgBlurringDao).orgBlurringExists(sor, "SOR");
        verify(orgBlurringDao).insert(Mockito.any());
    }

    @Test
    public void createOrgBlurring_sorDoesNotExist_shouldFail() throws SoresClientException {
        // Arrange
        when(soresClient.checkSorExists("12345678")).thenReturn(false);

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class, () -> {
            orgBlurringService.createOrgBlurring("123", "12345678", "SOR");
        });

        // Assert
        assertEquals(ErrorMessages.sorDoesNotExist(), exception.getMessage());
    }

    @Test
    public void createOrgBlurring_registrationAlreadyExists_shouldFail() {
        // Arrange
        when(orgBlurringDao.orgBlurringExists("123", "SOR")).thenReturn(true);

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class, () -> {
            orgBlurringService.createOrgBlurring("12345678", "123", "SOR");
        });

        // Assert
        assertEquals(ErrorMessages.orgBlurringAlreadyExists(), exception.getMessage());
    }

    @Test
    public void listOrgBlurringsForCVR_shouldReturnList() throws IdsasServiceException {
        // Arrange
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId("123");
        orgBlurringEntity.setOrganisationIdType("SOR");
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getAllOrgBlurrings("123")).thenReturn(orgBlurringEntities);

        // Act
        List<OrganizationIdDto> result = orgBlurringService.listOrgBlurringsForCVR("123");

        // Assert
        assertEquals(1, result.size());
        assertEquals("123", result.get(0).id);
    }

    @Test
    public void listOrgBlurringsForCVR_shouldReturnEmptyList() throws IdsasServiceException {
        // Arrange
        when(orgBlurringDao.getAllOrgBlurrings("123")).thenReturn(new ArrayList<>());

        // Act
        List<OrganizationIdDto> result = orgBlurringService.listOrgBlurringsForCVR("123");

        // Assert
        assertEquals(0, result.size());
    }

    @Test
    public void listOrgBlurringsForCVR_shouldThrowException() {
        // Arrange
        when(orgBlurringDao.getAllOrgBlurrings("123")).thenThrow(new DataAccessException("fejl") {
            @Override
            public String getMessage() {
                return super.getMessage();
            }
        });

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class, () -> {
            orgBlurringService.listOrgBlurringsForCVR("123");
        });

        // Assert
        assertEquals(ErrorMessages.daoAccessError(), exception.getMessage());
    }

    @Test
    public void listAllActiveOrgBlurrings_shouldReturnList() throws IdsasServiceException {
        // Arrange
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId("123");
        orgBlurringEntity.setOrganisationIdType("SOR");
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getAllActiveOrgBlurrings()).thenReturn(orgBlurringEntities);

        // Act
        List<OrganizationIdDto> result = orgBlurringService.listAllActiveOrgBlurrings();

        // Assert
        assertEquals(1, result.size());
        assertEquals("123", result.get(0).id);
    }

    @Test
    public void listAllActiveOrgBlurrings_shouldReturnEmptyList() throws IdsasServiceException {
        // Arrange
        when(orgBlurringDao.getAllActiveOrgBlurrings()).thenReturn(new ArrayList<>());

        // Act
        List<OrganizationIdDto> result = orgBlurringService.listAllActiveOrgBlurrings();

        // Assert
        assertEquals(0, result.size());
    }

    @Test
    public void listAllActiveOrgBlurrings_shouldThrowException() {
        // Arrange
        when(orgBlurringDao.getAllActiveOrgBlurrings()).thenThrow(new DataAccessException("fejl") {
            @Override
            public String getMessage() {
                return super.getMessage();
            }
        });

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class, () -> {
            orgBlurringService.listAllActiveOrgBlurrings();
        });

        // Assert
        assertEquals(ErrorMessages.daoAccessError(), exception.getMessage());
    }

    @Test
    public void removeOrgBlurring_shouldRemove() throws IdsasServiceException {
        // Arrange
        String cvr = "123";
        String sor = "12345678";
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId(sor);
        orgBlurringEntity.setOrganisationIdType("SOR");
        orgBlurringEntity.setCreatorCvr(cvr);
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getActiveOrgBlurrings(sor, "SOR")).thenReturn(orgBlurringEntities);

        // Act
        orgBlurringService.removeOrgBlurring(cvr, sor, "SOR");

        // Assert
        verify(orgBlurringDao).removeOrgBlurring(sor, "SOR");
    }

    @Test
    public void removeOrgBlurring_wrongCreatorCvr_throwsException() {
        // Arrange
        String cvr = "123";
        String sor = "12345678";
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId(sor);
        orgBlurringEntity.setOrganisationIdType("SOR");
        orgBlurringEntity.setCreatorCvr("456");
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getActiveOrgBlurrings(sor, "SOR")).thenReturn(orgBlurringEntities);

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class,
                () -> orgBlurringService.removeOrgBlurring(cvr, sor, "SOR"));

        // Assert
        assertEquals(ErrorMessages.orgBlurringNotCreatedByOrg(), exception.getMessage());
    }

    @Test
    public void removeOrgBlurring_noOrgBlurring_throwsException() {
        // Arrange
        String cvr = "123";
        String sor = "12345678";
        when(orgBlurringDao.getActiveOrgBlurrings(sor, "SOR")).thenReturn(new ArrayList<>());

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class,
                () -> orgBlurringService.removeOrgBlurring(cvr, sor, "SOR"));

        // Assert
        assertEquals(ErrorMessages.orgBlurringDoesNotExist(), exception.getMessage());
    }

    @Test
    public void removeOrgBlurring_daoException_throwsException() {
        // Arrange
        String cvr = "123";
        String sor = "12345678";
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId(sor);
        orgBlurringEntity.setOrganisationIdType("SOR");
        orgBlurringEntity.setCreatorCvr(cvr);
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getActiveOrgBlurrings(sor, "SOR")).thenReturn(orgBlurringEntities);
        Mockito.doThrow(new DataAccessException("fejl") {
            @Override
            public String getMessage() {
                return super.getMessage();
            }
        }).when(orgBlurringDao).removeOrgBlurring(sor, "SOR");

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class,
                () -> orgBlurringService.removeOrgBlurring(cvr, sor, "SOR"));

        // Assert
        assertEquals(ErrorMessages.daoAccessError(), exception.getMessage());
    }

}
